public class MertonJumpDifOptPriceGreeks
{

  // Declaration of the Native (C) function
  private native int s30jbc(char calput, int m, int n, double[] x, double s,
                            double[] t, double sigma, double r, double lambda, 
                            double jvol, double[] p, double[] delta,  
                            double[] gamma, double[] vega, double[] theta,
                            double[] rho, double[] vanna, double[] charm, 
                            double[] speed,  double[] colour, double[] zomma,
                            double[] vomma);

  static
    {
      // The runtime system executes a class's static
      // initializer when it loads the class.
      System.loadLibrary("nagCJavaInterface");
    }

  // The main program
  public static void main(String[] args)
    {
      double x[], t[], p[], delta[], gamma[], vega[], theta[], rho[], vanna[],
             charm[], speed[], colour[], zomma[], vomma[];
      double s, sigma, r, lambda, jvol;
      int i, j, m, n, retCode;
      char calput;

      // Create an object of class MertonJumpDifOptPriceGreeks
      MertonJumpDifOptPriceGreeks price = new MertonJumpDifOptPriceGreeks();

      calput = 'C';
      lambda = 5.0;
      s = 100.0;
      sigma = 0.25;
      r = 0.08;
      jvol = 0.25;
      m = 2;
      n = 1;

      p = new double[m*n];
      delta = new double[m*n];
      gamma = new double[m*n];
      vega = new double[m*n];
      theta = new double[m*n];
      rho = new double[m*n];
      vanna = new double[m*n];
      charm = new double[m*n];
      speed = new double[m*n];
      colour = new double[m*n];
      zomma = new double[m*n];
      vomma = new double[m*n];
      x = new double[m];
      t = new double[n];

      x[0] = 80.0;
      x[1] = 90.0;
      t[0] = 0.5;

      System.out.println();
      System.out.println("Call of NAG Merton jump-diffusion model option pricing with Greeks routine s30jbc");
      System.out.println();

      // Call method s30jbc of object price
      retCode = price.s30jbc(calput, m, n, x, s, t, sigma, r, lambda, jvol, p,
                             delta, gamma, vega, theta, rho, vanna, 
                             charm, speed, colour, zomma, vomma);

      System.out.print("Return code from s30jbc = ");
      System.out.println(retCode);
      System.out.println();

      if (retCode == 0)
        {
          // Print the input values
          System.out.println("European Call:");
          System.out.println("Spot " + s);
          System.out.println("Volatility " + sigma);
          System.out.println("Rate " + r);
          System.out.println("Jumps " + lambda);
          System.out.println("Jump vol " + jvol);
          System.out.println();

          // Print the solution
          for (j = 0; j < n; j++)
            {
              System.out.println("Time to Expiry : " + t[j]);
              System.out.print(" Strike     Price     Delta     Gamma    ");
              System.out.println("Vega      Theta      Rho");
              for (i = 0; i < m; i++)
                {
                  System.out.format("%7.4f   %7.4f   %7.4f   ",x[i],p[i*n + j],delta[i*n + j]);
                  System.out.format("%7.4f   %7.4f",gamma[i*n + j],vega[i*n + j]);
                  System.out.format("   %7.4f   %7.4f%n",theta[i*n + j],rho[i*n + j]);
                }
              System.out.println();
              System.out.print(" Strike     Price     Vanna    Charm     Speed");
              System.out.println("     Colour    Zomma      Vomma");
              for (i = 0; i < m; i++)
                {
                  System.out.format("%7.4f   %7.4f   %7.4f   ",x[i],p[i*n + j],vanna[i*n + j]);
                  System.out.format("%7.4f   %7.4f",charm[i*n + j],speed[i*n + j]);
                  System.out.format("   %7.4f   %7.4f   %7.4f%n",colour[i*n + j],zomma[i*n + j],vomma[i*n + j]);
                }
              System.out.println();
            }
        }
    }
}
